{% load i18n %}
<!DOCTYPE HTML>
<html lang="en">
	<head>
		<title>429</title>
		<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
	</head>
	<body>
		<h2 class="too-many-requests-429">{% trans 'Too Many Requests (429)' %}</h2>
		<p>{{ description }}</p>
	</body>
</html>
